@@ -3,7 +3,7 @@
 
 from amplitude.client import Amplitude
 from amplitude.event import BaseEvent, EventOptions, Identify, Revenue, IdentifyEvent, \
-    GroupIdentifyEvent, RevenueEvent, Plan
+    GroupIdentifyEvent, RevenueEvent, Plan, IngestionMetadata
 from amplitude.config import Config
 from amplitude.constants import PluginType
 from amplitude.plugin import EventPlugin, DestinationPlugin
