@@ -8,7 +8,7 @@ import logging
 from typing import Optional, Callable
 
 from amplitude import constants
-from amplitude.event import BaseEvent, Plan
+from amplitude.event import BaseEvent, Plan, IngestionMetadata
 from amplitude.storage import InMemoryStorageProvider, StorageProvider, Storage
 
 
@@ -33,6 +33,7 @@ class Config:
         storage_provider (amplitude.storage.StorageProvider, optional): Default to InMemoryStorageProvider.
             Provide storage instance for events buffer.
         plan (amplitude.event.Plan, optional): Tracking plan information. Default to None.
+        ingestion_metadata (amplitude.event.IngestionMetadata, optional): Ingestion metadata. Default to None.
 
     Properties:
         options: A dictionary contains minimum id length information. None if min_id_length not set.
@@ -55,7 +56,8 @@ class Config:
                  use_batch: bool = False,
                  server_url: Optional[str] = None,
                  storage_provider: StorageProvider = InMemoryStorageProvider(),
-                 plan: Plan = None):
+                 plan: Plan = None,
+                 ingestion_metadata: IngestionMetadata = None):
         """The constructor of Config class"""
         self.api_key: str = api_key
         self._flush_queue_size: int = flush_queue_size
@@ -71,6 +73,7 @@ class Config:
         self.storage_provider: StorageProvider = storage_provider
         self.opt_out: bool = False
         self.plan: Plan = plan
+        self.ingestion_metadata: IngestionMetadata = ingestion_metadata
 
     def get_storage(self) -> Storage:
         """Use configured StorageProvider to create a Storage instance then return.
